/*
 * ************************************
 *  scoring
 * ************************************
 * 
 *  Created on: 12.12.2005
 *  Author: 	Harald Wendel
 *
 *  Revision:   $Revision: 1.8 $
 * 
 */
package scoring;

import java.util.ArrayList;

import scoring.index.Document;
import scoring.index.Index;
import scoring.index.Query;
import scoring.index.Term;

/**
 * @author hardi
 *
 */
public class VectorSpaceScorer implements Scorer {

	public float score(Index index, int docId, boolean weighted, 
			Query query, Description desc) {
		
		/*
		 * dlen: Betrag des Dokumentvektors
		 * qlen: Betrag des Anfragevektors
		 */
		float dlen = 0.0f, qlen = 0.0f; 
		
		/*
		 * Anzahl der Dokumente im Index
		 */
		float docCount = index.docCount();
		
		/*
		 * zu bewertendes Dokument
		 */
		Document doc = index.getDoc(docId);

		/*
		 * dWeights: Vektor der Gewichte der Terme im Dokument
		 * qWeights: Vektor der Gewichte der Terme in der Anfrage  
		 */
		ArrayList<Float> qWeights = new ArrayList<Float>(); 
		ArrayList<Float> dWeights = new ArrayList<Float>();
		ArrayList<Float> qtfs = new ArrayList<Float>(); 
		ArrayList<Float> dtfs = new ArrayList<Float>();
		
		/*
		 * fr alle Terme im Dokument
		 */
		for (Term t : doc.terms()) {
			float dtf = t.tf(docId);
			if (weighted) {
				dtf = dtf / (float)doc.getMaxTermCount();
			}
			/*
			 * berechne das Gewicht aus:
			 * 
			 * 		dtf 	 - Frequenz des Terms im Dokument
			 * 		docCount - Anzahl der Dokumente im Index
			 * 		t.df()	 - Anzahl der Dokument die den Term enthalten
			 */
			float dw = weight(dtf, docCount, t.df());
			
			/*
			 * fge das Quadrat des Gewichts zum Betrag
			 * des Vektors hinzu
			 */
			dlen += dw * dw;
		}
			
		/*
		 * fr alle Terme in der Anfrage
		 */
		for (Term t : query.terms()) {
			
			float dtf = t.tf(docId);
			float qtf = query.tf(t);
			
			if (weighted) {
				dtf = dtf / (float)doc.getMaxTermCount();
				qtf = (float)(0.5 + 0.5 * qtf / (float)query.getMaxTermCount());
				qtf = qtf / (float)query.getMaxTermCount();
			}
			
			/*
			 * berechne das Gewicht 
			 * (analog zur Berechnung des Gewichts im
			 *  Dokumentenvektor)
			 */
			float dw = weight(dtf, docCount, t.df());
			float qw = weight(qtf, docCount, t.df());
			qlen += qw * qw;
				
			dWeights.add(qw);
			qWeights.add(dw);
			dtfs.add(dtf);
			qtfs.add(qtf);
		}
		
		/*
		 * Die Betrge der Vektoren
		 */
		dlen = (float)Math.sqrt(dlen);
		qlen = (float)Math.sqrt(qlen);
		
		/*
		 * Das Skalarprodukt der beiden Vektoren
		 */
		float dotp = dot(qWeights, dWeights);
		
		if (desc != null) {
			desc.setQuery(query);
			desc.setDocumentVectorLenght(dlen);
			desc.setQueryVectorLength(qlen);
			desc.setDotProduct(dotp);
			desc.setDocumentWeights(dWeights.toArray(new Float[] {}));
			desc.setQueryWeights(qWeights.toArray(new Float[] {}));
			desc.setQueryFrequencies(qtfs.toArray(new Float[] {}));
			desc.setDocumentFrequencies(dtfs.toArray(new Float[] {}));
		}
		
		if (Math.abs(0 - dotp) < 0.0000001) {
			return 0;
		}
	
		/*
		 * Bewertung:
		 * 
		 * 		Skalarprodukt / (Lnge Dokumentvektor * Lnge Anfragevektor)
		 */
		return dotp / (dlen * qlen);
	}
	
	private float weight(float tf, float docCount, float df) {
		return (float)(tf * Math.log10(docCount / df));
	}
	
	private float dot(ArrayList<Float> v1, ArrayList<Float> v2) {
		float scal = 0.0f;
		for (int i = 0; i < v1.size(); i++) {
			scal += v1.get(i) * v2.get(i);
		}	
		return scal;	
	}

}
