/*
 * ************************************
 *  scoring
 * ************************************
 * 
 *  Created on: 12.12.2005
 *  Author: 	Harald Wendel
 *
 *  Revision:   $Revision$
 * 
 */
package scoring.index;

import java.io.IOException;
import java.util.ArrayList;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import scoring.Analyzer;
import scoring.Description;
import scoring.Hit;
import scoring.HitComparator;
import scoring.Scorer;
import scoring.VectorSpaceScorer;

/**
 * @author hardi
 *
 */
public class Index {
	
	private static Index instance = null;

	public static Index getInstance() {
		if (instance == null) {
			instance = new Index();
		}
		return instance;
	}
	
	private SortedMap<String, Term>	terms;
	private ArrayList<Document>		docs;
	private Scorer					scorer;
	

	private Index() {
		scorer = new VectorSpaceScorer();
		clear();
	}
	
	public void clear() {
		terms = new TreeMap<String, Term>(String.CASE_INSENSITIVE_ORDER);
		docs = new ArrayList<Document>();
	}
		
	public void add(Document doc, Analyzer a) throws IOException {
		
		int docId = docs.size();
		
		Tokenizer tok = new Tokenizer(doc.r);
		
		while(tok.hasMoreTokens()) {
			
			String token = tok.nextToken();
			
			if (a != null && a.contains(token)) {
				continue;
			}
			
			Term term = terms.get(token);
			
			if (term == null) {
				term = new Term(token);
				terms.put(token, term);
			}
			
			term.addDoc(docId);
			doc.addTerm(term);
		}
		
		docs.add(docId, doc);
	}
	
	public Term get(String term) {
		return terms.get(term);
	}
	
	public Term[] terms() {
		return terms.keySet().toArray(new Term[] {});
	}
	
	public int docCount() {
		return docs.size();
	}
	
	public Document getDoc(int docId) {
		if (docId >= 0 && docId < docCount()) {
			return docs.get(docId);
		}
		return null;
	}
	
	/**
	 * Fhrt eine Volltextsuche durch
	 * 
	 * @param query 	Anfrage
	 * @param a 		Stopwort-Analysierer
	 * @param weighted 	soll die Bewertung mit gewichteten Termen erfolgen?
	 * @return			sortierte Menge von Treffern
	 */
	public SortedSet<Hit> search(Query query, Analyzer a, boolean weighted) {
			
		/*
		 * Ausfhren der Anfrage:
		 * 
		 * 		- Anfragestring wird in Token zerlegt
		 * 
		 * 		- mit Hilfe des Analyzers wird berprft,
		 * 		  ob es sich um ein Stoppwort handelt
		 * 
		 *  	- wenn nicht wird aus dem Index eine Referenz 
		 *        auf den Term auf den Term geholt
		 *        
		 *        			Term t = index.get(token);
		 *        
		 *      - wenn der Term im Index vorhanden ist (t != null)
		 *        dann wird der Term zur Liste der Anfrageterme
		 *        hinzugefgt	 	 
		 */
		query.execute(this, a);
		
		/*
		 * Menge aller Dokumente, die zu der Anfrage passen, bzw.
		 * mindestens einen Term der Anfrage enthalten 
		 */
		SortedSet<Integer> retrievedDocs = new TreeSet<Integer>(new IntComparator());
			
		/*
		 * fr alle Terme der Anfrage
		 */
		for (Term t : query.terms()) {
			
			/*
			 * fr alle Dokumente die den Term enthalten
			 */
			for (int docId : t.getDocs()) {
				
				/*
				 * Wenn das Dokument noch nicht in der Menge der
				 * gefundenen Dokumente entahlten ist, dann wird
				 * es hinzugefgt
				 */
				if (!retrievedDocs.contains(docId)) {
					retrievedDocs.add(docId);
				}
			}
		}
		
		/*
		 * Menge aller Treffer, geordnet nach ihrem Rang
		 */
		SortedSet<Hit> hitSet = new TreeSet<Hit>(new HitComparator());
		
		/*
		 * fr alle gefundenen Dokumente
		 */
		for (int doc : retrievedDocs) {
			Description d = new Description();
			
			/*
			 * wird der Rang des Dokuments berechnet und ein neuer
			 * Treffer erzeugt
			 */
			hitSet.add(new Hit(doc, scorer.score(this, doc, weighted, query, d), d));
		}
		
		return hitSet;
	}

	public Scorer getScorer() {
		return scorer;
	}

	public void setScorer(Scorer scorer) {
		this.scorer = scorer;
	}
	
}
